@@ -23,18 +23,20 @@ NAME=$1
 shift
 #CC_OPTS="-DSQLITE_ENABLE_RTREE -DSQLITE_ENABLE_MEMSYS5"
 CC_OPTS="-DSQLITE_ENABLE_MEMSYS5"
+CC=gcc
 SPEEDTEST_OPTS="--shrink-memory --reprepare --stats --heap 10000000 64"
 SIZE=5
 LEAN_OPTS="-DSQLITE_THREADSAFE=0"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_DEFAULT_MEMSTATUS=0"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_DEFAULT_WAL_SYNCHRONOUS=1"
-LEAN_OPTS="$LEAN_OPTS -DSQLITE_LIKE_DOESNT_MATCH_BLOB"
+LEAN_OPTS="$LEAN_OPTS -DSQLITE_LIKE_DOESNT_MATCH_BLOBS"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_MAX_EXPR_DEPTH=0"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_OMIT_DECLTYPE"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_OMIT_DEPRECATED"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_OMIT_PROGRESS_CALLBACK"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_OMIT_SHARED_CACHE"
 LEAN_OPTS="$LEAN_OPTS -DSQLITE_USE_ALLOCA"
+BASELINE="trunk"
 doExplain=0
 doCachegrind=1
 while test "$1" != ""; do
@@ -80,11 +82,25 @@ while test "$1" != ""; do
     --lean)
         CC_OPTS="$CC_OPTS $LEAN_OPTS"
         ;;
+    --clang)
+        CC=clang
+        ;;
+    --icc)
+        CC=/home/drh/intel/bin/icc
+        ;;
+    --gcc7)
+        CC=gcc-7
+        ;;
     --heap)
         CC_OPTS="$CC_OPTS -DSQLITE_ENABLE_MEMSYS5"
         shift;
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS --heap $1 64"
         ;;
+    --lookaside)
+        shift;
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --lookaside $1 $2"
+        shift;
+        ;;
     --repeat)
         CC_OPTS="$CC_OPTS -DSQLITE_ENABLE_RCACHE"
         shift;
@@ -94,9 +110,25 @@ while test "$1" != ""; do
         shift;
         SPEEDTEST_OPTS="$SPEEDTEST_OPTS --mmap $1"
         ;;
-    *)
+    --rtree)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --testset rtree"
+        CC_OPTS="$CC_OPTS -DSQLITE_ENABLE_RTREE"
+        ;;
+    --orm)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --testset orm"
+        ;;
+    --cte)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --testset cte"
+        ;;
+    --fp)
+        SPEEDTEST_OPTS="$SPEEDTEST_OPTS --testset fp"
+        ;;
+    -*)
         CC_OPTS="$CC_OPTS $1"
         ;;
+    *)
+	BASELINE=$1
+        ;;
   esac
   shift
 done
@@ -105,15 +137,15 @@ echo "NAME           = $NAME" | tee summary-$NAME.txt
 echo "SPEEDTEST_OPTS = $SPEEDTEST_OPTS" | tee -a summary-$NAME.txt
 echo "CC_OPTS        = $CC_OPTS" | tee -a summary-$NAME.txt
 rm -f cachegrind.out.* speedtest1 speedtest1.db sqlite3.o
-gcc -g -Os -Wall -I. $CC_OPTS -c sqlite3.c
+$CC -g -Os -Wall -I. $CC_OPTS -c sqlite3.c
 size sqlite3.o | tee -a summary-$NAME.txt
 if test $doExplain -eq 1; then
-  gcc -g -Os -Wall -I. $CC_OPTS \
+  $CC -g -Os -Wall -I. $CC_OPTS \
      -DSQLITE_ENABLE_EXPLAIN_COMMENTS \
     ./shell.c ./sqlite3.c -o sqlite3 -ldl -lpthread
 fi
 SRC=./speedtest1.c
-gcc -g -Os -Wall -I. $CC_OPTS $SRC ./sqlite3.o -o speedtest1 -ldl -lpthread
+$CC -g -Os -Wall -I. $CC_OPTS $SRC ./sqlite3.o -o speedtest1 -ldl -lpthread
 ls -l speedtest1 | tee -a summary-$NAME.txt
 if test $doCachegrind -eq 1; then
   valgrind --tool=cachegrind ./speedtest1 speedtest1.db \
@@ -125,7 +157,12 @@ size sqlite3.o | tee -a summary-$NAME.txt
 wc sqlite3.c
 if test $doCachegrind -eq 1; then
   cg_anno.tcl cachegrind.out.* >cout-$NAME.txt
+  echo '*****************************************************' >>cout-$NAME.txt
+  sed 's/^[0-9=-]\{9\}/==00000==/' summary-$NAME.txt >>cout-$NAME.txt
 fi
 if test $doExplain -eq 1; then
   ./speedtest1 --explain $SPEEDTEST_OPTS | ./sqlite3 >explain-$NAME.txt
 fi
+if test "$NAME" != "$BASELINE"; then
+  fossil test-diff --tk -c 20 cout-$BASELINE.txt cout-$NAME.txt
+fi
